@@ -10,7 +10,6 @@
 
 #include <clutter/x11/clutter-x11.h>
 
-#include "../core/window-private.h" /* FIXME: add meta_window_is_override_redirect */
 #include "display.h"
 #include "errors.h"
 #include "frame.h"
@@ -48,6 +47,8 @@ struct _MutterWindowPrivate
    * texture */
   GdkRegion        *bounding_region;
 
+  gint              freeze_count;
+
   /*
    * These need to be counters rather than flags, since more plugins
    * can implement same effect; the practicality of stacking effects
@@ -66,7 +67,10 @@ struct _MutterWindowPrivate
   guint		    disposed               : 1;
   guint             redecorating           : 1;
 
-  guint		    needs_repair           : 1;
+  guint		    needs_damage_all       : 1;
+  guint		    received_damage        : 1;
+
+  guint		    needs_pixmap           : 1;
   guint		    needs_reshape          : 1;
   guint		    size_changed           : 1;
 
@@ -273,9 +277,6 @@ mutter_meta_window_decorated_notify (MetaWindow *mw,
       priv->damage = None;
     }
 
-  g_hash_table_remove (info->windows_by_xid, (gpointer) priv->xwindow);
-  g_hash_table_insert (info->windows_by_xid, (gpointer) new_xwindow, self);
-
   g_free (priv->desc);
   priv->desc = NULL;
 
@@ -336,7 +337,8 @@ mutter_window_constructed (GObject *object)
   if (priv->attrs.class == InputOnly)
     priv->damage = None;
   else
-    priv->damage = XDamageCreate (xdisplay, xwindow, XDamageReportNonEmpty);
+    priv->damage = XDamageCreate (xdisplay, xwindow,
+                                  XDamageReportBoundingBox);
 
   format = XRenderFindVisualFormat (xdisplay, priv->attrs.visual);
 
@@ -422,7 +424,6 @@ mutter_window_dispose (GObject *object)
     }
 
   info->windows = g_list_remove (info->windows, (gconstpointer) self);
-  g_hash_table_remove (info->windows_by_xid, (gpointer) priv->xwindow);
 
   /*
    * Release the extra reference we took on the actor.
@@ -701,10 +702,7 @@ mutter_window_get_window_type (MutterWindow *self)
 gboolean
 mutter_window_is_override_redirect (MutterWindow *self)
 {
-  if (self->priv->window->override_redirect)
-    return TRUE;
-
-  return FALSE;
+  return meta_window_is_override_redirect (self->priv->window);
 }
 
 const char *mutter_window_get_description (MutterWindow *self)
@@ -726,6 +724,18 @@ const char *mutter_window_get_description (MutterWindow *self)
   return self->priv->desc;
 }
 
+/**
+ * mutter_window_get_workspace:
+ * @self: #MutterWindow
+ *
+ * Returns the index of workspace on which this window is located; if the
+ * window is sticky, or is not currently located on any workspace, returns -1.
+ * This function is deprecated  and should not be used in newly written code;
+ * meta_window_get_workspace() instead.
+ *
+ * Return value: (transfer none): index of workspace on which this window is
+ * located.
+ */
 gint
 mutter_window_get_workspace (MutterWindow *self)
 {
@@ -742,6 +752,9 @@ mutter_window_get_workspace (MutterWindow *self)
 
   workspace = meta_window_get_workspace (priv->window);
 
+  if (!workspace)
+    return -1;
+
   return meta_workspace_index (workspace);
 }
 
@@ -758,6 +771,60 @@ mutter_window_showing_on_its_workspace (MutterWindow *self)
   return meta_window_showing_on_its_workspace (self->priv->window);
 }
 
+static void
+mutter_window_freeze (MutterWindow *self)
+{
+  self->priv->freeze_count++;
+}
+
+static void
+mutter_window_damage_all (MutterWindow *self)
+{
+  MutterWindowPrivate *priv = self->priv;
+  ClutterX11TexturePixmap *texture_x11 = CLUTTER_X11_TEXTURE_PIXMAP (priv->actor);
+  guint pixmap_width = 0;
+  guint pixmap_height = 0;
+
+  if (!priv->needs_damage_all)
+    return;
+
+  g_object_get (texture_x11,
+                "pixmap-width", &pixmap_width,
+                "pixmap-height", &pixmap_height,
+                NULL);
+
+  clutter_x11_texture_pixmap_update_area (texture_x11,
+                                          0,
+                                          0,
+                                          pixmap_width,
+                                          pixmap_height);
+
+  priv->needs_damage_all = FALSE;
+}
+
+static void
+mutter_window_thaw (MutterWindow *self)
+{
+  self->priv->freeze_count--;
+
+  if (G_UNLIKELY (self->priv->freeze_count < 0))
+    {
+      g_warning ("Error in freeze/thaw accounting.");
+      self->priv->freeze_count = 0;
+      return;
+    }
+
+  if (self->priv->freeze_count)
+    return;
+
+  /* Since we ignore damage events while a window is frozen for certain effects
+   * we may need to issue an update_area() covering the whole pixmap if we
+   * don't know what real damage has happened. */
+
+  if (self->priv->needs_damage_all)
+    mutter_window_damage_all (self);
+}
+
 gboolean
 mutter_window_effect_in_progress (MutterWindow *self)
 {
@@ -769,11 +836,11 @@ mutter_window_effect_in_progress (MutterWindow *self)
 }
 
 static void
-mutter_window_mark_for_repair (MutterWindow *self)
+mutter_window_queue_create_pixmap (MutterWindow *self)
 {
   MutterWindowPrivate *priv = self->priv;
 
-  priv->needs_repair = TRUE;
+  priv->needs_pixmap = TRUE;
 
   if (!priv->mapped)
     return;
@@ -789,6 +856,21 @@ mutter_window_mark_for_repair (MutterWindow *self)
   clutter_actor_queue_redraw (priv->actor);
 }
 
+static gboolean
+is_freeze_thaw_effect (gulong event)
+{
+  switch (event)
+  {
+  case MUTTER_PLUGIN_DESTROY:
+  case MUTTER_PLUGIN_MAXIMIZE:
+  case MUTTER_PLUGIN_UNMAXIMIZE:
+    return TRUE;
+    break;
+  default:
+    return FALSE;
+  }
+}
+
 static gboolean
 start_simple_effect (MutterWindow *self,
                      gulong        event)
@@ -796,6 +878,7 @@ start_simple_effect (MutterWindow *self,
   MutterWindowPrivate *priv = self->priv;
   MetaCompScreen *info = meta_screen_get_compositor_data (priv->screen);
   gint *counter = NULL;
+  gboolean use_freeze_thaw = FALSE;
 
   if (!info->plugin_mgr)
     return FALSE;
@@ -820,6 +903,11 @@ start_simple_effect (MutterWindow *self,
 
   g_assert (counter);
 
+  use_freeze_thaw = is_freeze_thaw_effect (event);
+
+  if (use_freeze_thaw)
+    mutter_window_freeze (self);
+
   (*counter)++;
 
   if (!mutter_plugin_manager_event_simple (info->plugin_mgr,
@@ -827,6 +915,8 @@ start_simple_effect (MutterWindow *self,
                                            event))
     {
       (*counter)--;
+      if (use_freeze_thaw)
+        mutter_window_thaw (self);
       return FALSE;
     }
 
@@ -847,10 +937,10 @@ mutter_window_after_effects (MutterWindow *self)
   mutter_window_sync_visibility (self);
   mutter_window_sync_actor_position (self);
 
-  if (!priv->window->mapped)
+  if (!meta_window_is_mapped (priv->window))
     mutter_window_detach (self);
 
-  if (priv->needs_repair)
+  if (priv->needs_pixmap)
     clutter_actor_queue_redraw (priv->actor);
 }
 
@@ -919,6 +1009,9 @@ mutter_window_effect_completed (MutterWindow *self,
     break;
   }
 
+  if (is_freeze_thaw_effect (event))
+    mutter_window_thaw (self);
+
   if (!mutter_window_effect_in_progress (self))
     mutter_window_after_effects (self);
 }
@@ -944,7 +1037,7 @@ mutter_window_detach (MutterWindow *self)
   clutter_x11_texture_pixmap_set_pixmap (CLUTTER_X11_TEXTURE_PIXMAP (priv->actor),
                                          None);
 
-  mutter_window_mark_for_repair (self);
+  mutter_window_queue_create_pixmap (self);
 }
 
 void
@@ -965,7 +1058,6 @@ mutter_window_destroy (MutterWindow *self)
    */
   info = meta_screen_get_compositor_data (priv->screen);
   info->windows = g_list_remove (info->windows, (gconstpointer) self);
-  g_hash_table_remove (info->windows_by_xid, (gpointer)priv->xwindow);
 
   if (priv->type == META_COMP_WINDOW_DROPDOWN_MENU ||
       priv->type == META_COMP_WINDOW_POPUP_MENU ||
@@ -1008,7 +1100,7 @@ mutter_window_sync_actor_position (MutterWindow *self)
       priv->attrs.height != window_rect.height)
     {
       priv->size_changed = TRUE;
-      mutter_window_mark_for_repair (self);
+      mutter_window_queue_create_pixmap (self);
     }
 
   /* XXX deprecated: please use meta_window_get_outer_rect instead */
@@ -1125,6 +1217,7 @@ mutter_window_maximize (MutterWindow       *self,
   clutter_actor_set_size (CLUTTER_ACTOR (self), old_rect->width, old_rect->height);
 
   self->priv->maximize_in_progress++;
+  mutter_window_freeze (self);
 
   if (!info->plugin_mgr ||
       !mutter_plugin_manager_event_maximize (info->plugin_mgr,
@@ -1135,6 +1228,7 @@ mutter_window_maximize (MutterWindow       *self,
 
     {
       self->priv->maximize_in_progress--;
+      mutter_window_thaw (self);
     }
 }
 
@@ -1152,6 +1246,7 @@ mutter_window_unmaximize (MutterWindow      *self,
   clutter_actor_set_size (CLUTTER_ACTOR (self), old_rect->width, old_rect->height);
 
   self->priv->unmaximize_in_progress++;
+  mutter_window_freeze (self);
 
   if (!info->plugin_mgr ||
       !mutter_plugin_manager_event_maximize (info->plugin_mgr,
@@ -1161,6 +1256,7 @@ mutter_window_unmaximize (MutterWindow      *self,
 					     new_rect->width, new_rect->height))
     {
       self->priv->unmaximize_in_progress--;
+      mutter_window_thaw (self);
     }
 }
 
@@ -1186,7 +1282,7 @@ mutter_window_new (MetaWindow *window)
 
   /* FIXME: Remove the redundant data we store in self->priv->attrs, and
    * simply query metacity core for the data. */
-  if (!XGetWindowAttributes (display->xdisplay, top_window, &attrs))
+  if (!XGetWindowAttributes (meta_display_get_xdisplay (display), top_window, &attrs))
     return NULL;
 
   self = g_object_new (MUTTER_TYPE_COMP_WINDOW,
@@ -1199,6 +1295,8 @@ mutter_window_new (MetaWindow *window)
   priv = self->priv;
 
   priv->mapped = meta_window_toplevel_is_mapped (priv->window);
+  if (priv->mapped)
+    mutter_window_queue_create_pixmap (self);
 
   mutter_window_sync_actor_position (self);
 
@@ -1213,7 +1311,6 @@ mutter_window_new (MetaWindow *window)
    * before we first paint.
    */
   info->windows = g_list_append (info->windows, self);
-  g_hash_table_insert (info->windows_by_xid, (gpointer) top_window, self);
 
   return self;
 }
@@ -1227,7 +1324,7 @@ mutter_window_mapped (MutterWindow *self)
 
   priv->mapped = TRUE;
 
-  mutter_window_mark_for_repair (self);
+  mutter_window_queue_create_pixmap (self);
 }
 
 void
@@ -1243,7 +1340,7 @@ mutter_window_unmapped (MutterWindow *self)
     return;
 
   mutter_window_detach (self);
-  priv->needs_repair = FALSE;
+  priv->needs_pixmap = FALSE;
 }
 
 static void
@@ -1455,7 +1552,7 @@ mutter_window_reset_visible_regions (MutterWindow *self)
 }
 
 static void
-check_needs_repair (MutterWindow *self)
+check_needs_pixmap (MutterWindow *self)
 {
   MutterWindowPrivate *priv     = self->priv;
   MetaScreen          *screen   = priv->screen;
@@ -1466,7 +1563,7 @@ check_needs_repair (MutterWindow *self)
   Window               xwindow  = priv->xwindow;
   gboolean             full     = FALSE;
 
-  if (!priv->needs_repair)
+  if (!priv->needs_pixmap)
     return;
 
   if (!priv->mapped)
@@ -1538,74 +1635,52 @@ check_needs_repair (MutterWindow *self)
       full = TRUE;
     }
 
- /*
-   * TODO -- on some gfx hardware updating the whole texture instead of
-   * the individual rectangles is actually quicker, so we might want to
-   * make this a configurable option (on desktop HW with multiple pipelines
-   * it is usually quicker to just update the damaged parts).
-   *
-   * If we are using TFP we update the whole texture (this simply trigers
-   * the texture rebind).
-   */
-  if (full
-#ifdef HAVE_GLX_TEXTURE_PIXMAP
-      || (CLUTTER_GLX_IS_TEXTURE_PIXMAP (priv->actor) &&
-          clutter_glx_texture_pixmap_using_extension
-                  (CLUTTER_GLX_TEXTURE_PIXMAP (priv->actor)))
-#endif /* HAVE_GLX_TEXTURE_PIXMAP */
-      )
-    {
-      XDamageSubtract (xdisplay, priv->damage, None, None);
-
-      clutter_x11_texture_pixmap_update_area
-	(CLUTTER_X11_TEXTURE_PIXMAP (priv->actor),
-	 0,
-	 0,
-	 clutter_actor_get_width (priv->actor),
-	 clutter_actor_get_height (priv->actor));
-    }
-  else
-    {
-      XRectangle   *r_damage;
-      XRectangle    r_bounds;
-      XserverRegion parts;
-      int           i, r_count;
-
-      parts = XFixesCreateRegion (xdisplay, 0, 0);
-      XDamageSubtract (xdisplay, priv->damage, None, parts);
-
-      r_damage = XFixesFetchRegionAndBounds (xdisplay,
-					     parts,
-					     &r_count,
-					     &r_bounds);
-
-      if (r_damage)
-	{
-	  for (i = 0; i < r_count; ++i)
-	    {
-	      clutter_x11_texture_pixmap_update_area
-		(CLUTTER_X11_TEXTURE_PIXMAP (priv->actor),
-		 r_damage[i].x,
-		 r_damage[i].y,
-		 r_damage[i].width,
-		 r_damage[i].height);
-	    }
-	}
-
-      XFree (r_damage);
-      XFixesDestroyRegion (xdisplay, parts);
-    }
-
   meta_error_trap_pop (display, FALSE);
 
-  priv->needs_repair = FALSE;
+  priv->needs_pixmap = FALSE;
+}
+
+static gboolean
+is_frozen (MutterWindow *self)
+{
+  return self->priv->freeze_count ? TRUE : FALSE;
 }
 
 void
 mutter_window_process_damage (MutterWindow       *self,
 			      XDamageNotifyEvent *event)
 {
-  mutter_window_mark_for_repair (self);
+  MutterWindowPrivate *priv = self->priv;
+  ClutterX11TexturePixmap *texture_x11 = CLUTTER_X11_TEXTURE_PIXMAP (priv->actor);
+
+  priv->received_damage = TRUE;
+
+  if (is_frozen (self))
+    {
+      /* The window is frozen due to an effect in progress: we ignore damage
+       * here on the off chance that this will stop the corresponding
+       * texture_from_pixmap from being update.
+       *
+       * needs_damage_all tracks that some unknown damage happened while the
+       * window was frozen so that when the window becomes unfrozen we can
+       * issue a full window update to cover any lost damage.
+       *
+       * It should be noted that this is an unreliable mechanism since it's
+       * quite likely that drivers will aim to provide a zero-copy
+       * implementation of the texture_from_pixmap extension and in those cases
+       * any drawing done to the window is always immediately reflected in the
+       * texture regardless of damage event handling.
+       */
+      priv->needs_damage_all = TRUE;
+      return;
+    }
+
+
+  clutter_x11_texture_pixmap_update_area (texture_x11,
+                                          event->area.x,
+                                          event->area.y,
+                                          event->area.width,
+                                          event->area.height);
 }
 
 void
@@ -1636,7 +1711,7 @@ check_needs_reshape (MutterWindow *self)
 #ifdef HAVE_SHAPE
   if (priv->shaped)
     {
-      Display *xdisplay = meta_display_get_xdisplay (priv->window->display);
+      Display *xdisplay = meta_display_get_xdisplay (meta_window_get_display (priv->window));
       XRectangle *rects;
       int n_rects, ordering;
 
@@ -1677,16 +1752,25 @@ void
 mutter_window_pre_paint (MutterWindow *self)
 {
   MutterWindowPrivate *priv = self->priv;
+  MetaScreen          *screen   = priv->screen;
+  MetaDisplay         *display  = meta_screen_get_display (screen);
+  Display             *xdisplay = meta_display_get_xdisplay (display);
 
-  /* The window is frozen due to a pending animation: we'll wait until
-   * the animation finishes to reshape and repair the window */
-  if (priv->destroy_in_progress    ||
-      priv->maximize_in_progress   ||
-      priv->unmaximize_in_progress)
-    return;
+  if (is_frozen (self))
+    {
+      /* The window is frozen due to a pending animation: we'll wait until
+       * the animation finishes to reshape and repair the window */
+      return;
+    }
+
+  if (priv->received_damage)
+    {
+      XDamageSubtract (xdisplay, priv->damage, None, None);
+      priv->received_damage = FALSE;
+    }
 
   check_needs_reshape (self);
-  check_needs_repair (self);
+  check_needs_pixmap (self);
 }
 
 void
