@@ -24,6 +24,8 @@
 #include <X11/Xatom.h>
 #include <unistd.h>
 
+#include "gtk-compat.h"
+
 static GtkWidget* do_appwindow (void);
 
 static gboolean aspect_on;
@@ -62,19 +64,22 @@ static void
 on_realize_set_struts (GtkWindow *window,
                        gpointer   data)
 {
+  GtkWidget *widget;
   int left;
   int right;
   int top;
   int bottom;
 
-  g_return_if_fail (GTK_WIDGET_REALIZED (window));
+  widget = GTK_WIDGET (window);
+
+  g_return_if_fail (gtk_widget_get_realized (widget));
 
   left = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (window), "meta-strut-left"));
   right = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (window), "meta-strut-right"));
   top = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (window), "meta-strut-top"));
   bottom = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (window), "meta-strut-bottom"));
   
-  set_gdk_window_struts (GTK_WIDGET (window)->window,
+  set_gdk_window_struts (gtk_widget_get_window (widget),
                          left, right, top, bottom);
 }
 
@@ -85,6 +90,10 @@ set_gtk_window_struts (GtkWidget  *window,
                        int         top,
                        int         bottom)
 {
+  GtkWidget *widget;
+
+  widget = GTK_WIDGET (window);
+
   g_object_set_data (G_OBJECT (window), "meta-strut-left",
                      GINT_TO_POINTER (left));
   g_object_set_data (G_OBJECT (window), "meta-strut-right",
@@ -93,18 +102,18 @@ set_gtk_window_struts (GtkWidget  *window,
                      GINT_TO_POINTER (top));
   g_object_set_data (G_OBJECT (window), "meta-strut-bottom",
                      GINT_TO_POINTER (bottom));
-  
+
   g_signal_handlers_disconnect_by_func (G_OBJECT (window),
                                         on_realize_set_struts,
                                         NULL);
-                                
+
   g_signal_connect_after (G_OBJECT (window),
                           "realize",
                           G_CALLBACK (on_realize_set_struts),
                           NULL);
 
-  if (GTK_WIDGET_REALIZED (window))
-    set_gdk_window_struts (GTK_WIDGET (window)->window,
+  if (gtk_widget_get_realized (widget))
+    set_gdk_window_struts (gtk_widget_get_window (widget),
                            left, right, top, bottom);
 }
 
@@ -129,15 +138,18 @@ static void
 on_realize_set_type (GtkWindow *window,
                      gpointer   data)
 {
+  GtkWidget *widget;
   const char *type;
 
-  g_return_if_fail (GTK_WIDGET_REALIZED (window));
+  widget =  GTK_WIDGET (window);
+
+  g_return_if_fail (gtk_widget_get_realized (widget));
 
   type = g_object_get_data (G_OBJECT (window), "meta-window-type");
 
   g_return_if_fail (type != NULL);
   
-  set_gdk_window_type (GTK_WIDGET (window)->window,
+  set_gdk_window_type (gtk_widget_get_window (widget),
                        type);
 }
 
@@ -145,6 +157,10 @@ static void
 set_gtk_window_type (GtkWindow  *window,
                      const char *type)
 {
+  GtkWidget *widget;
+
+  widget = GTK_WIDGET (window);
+
   g_object_set_data (G_OBJECT (window), "meta-window-type", (char*) type);
 
   g_signal_handlers_disconnect_by_func (G_OBJECT (window),
@@ -156,8 +172,8 @@ set_gtk_window_type (GtkWindow  *window,
                           G_CALLBACK (on_realize_set_type),
                           NULL);
 
-  if (GTK_WIDGET_REALIZED (window))
-    set_gdk_window_type (GTK_WIDGET (window)->window,
+  if (gtk_widget_get_realized (widget))
+    set_gdk_window_type (gtk_widget_get_window (widget),
                          type);
 }
 
@@ -171,14 +187,22 @@ static void
 on_realize_set_border_only (GtkWindow *window,
                             gpointer   data)
 {
-  g_return_if_fail (GTK_WIDGET_REALIZED (window));
+  GtkWidget *widget;
+
+  widget = GTK_WIDGET (window);
+
+  g_return_if_fail (gtk_widget_get_realized (widget));
   
-  set_gdk_window_border_only (GTK_WIDGET (window)->window);
+  set_gdk_window_border_only (gtk_widget_get_window (widget));
 }
 
 static void
 set_gtk_window_border_only (GtkWindow  *window)
 {
+  GtkWidget *widget;
+
+  widget = GTK_WIDGET (window);
+
   g_signal_handlers_disconnect_by_func (G_OBJECT (window),
                                         on_realize_set_border_only,
                                         NULL);
@@ -188,8 +212,8 @@ set_gtk_window_border_only (GtkWindow  *window)
                           G_CALLBACK (on_realize_set_border_only),
                           NULL);
 
-  if (GTK_WIDGET_REALIZED (window))
-    set_gdk_window_border_only (GTK_WIDGET (window)->window);
+  if (gtk_widget_get_realized (widget))
+    set_gdk_window_border_only (gtk_widget_get_window (widget));
 }
 
 int
@@ -285,7 +309,7 @@ response_cb (GtkDialog *dialog,
 }
 
 static void
-dialog_cb (gpointer             callback_data,
+dialog_cb (gpointer		callback_data,
            guint                callback_action,
            GtkWidget           *widget)
 {
@@ -812,6 +836,26 @@ destroy_cb (GtkWidget *w, gpointer data)
     gtk_main_quit ();
 }
 
+static void
+insert_stock_button (GtkWidget          *toolbar,
+                     const gchar        *stock_id,
+                     const gchar        *text,
+                     GCallback           callback,
+                     gpointer            user_data)
+{
+  GtkToolItem *button;
+
+  button = gtk_tool_button_new_from_stock (stock_id);
+  gtk_tool_item_set_tooltip_text (button, text);
+  g_signal_connect (G_OBJECT (button),
+                    "clicked",
+                    callback,
+                    user_data);
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar),
+                      button,
+                      -1); /*-1 means append to end of toolbar*/
+}
+
 static GtkWidget *
 do_appwindow (void)
 {
@@ -903,45 +947,25 @@ do_appwindow (void)
    */
   toolbar = gtk_toolbar_new ();
 
-  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
-                            GTK_STOCK_NEW,
-                            "Open another one of these windows",
-                            NULL,
-                            G_CALLBACK (do_appwindow),
-                            window, /* user data for callback */
-                            -1);  /* -1 means "append" */
-  
-  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
-                            GTK_STOCK_OPEN,
-                            "This is a demo button that locks up the demo",
-                            NULL,
-                            G_CALLBACK (sleep_cb),
-                            window, /* user data for callback */
-                            -1);  /* -1 means "append" */
+  insert_stock_button (toolbar, GTK_STOCK_NEW,
+                       "Open another one of these windows",
+                       G_CALLBACK (do_appwindow), window);
 
-  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
-                            GTK_STOCK_OPEN,
-                            "This is a demo button that toggles window decorations",
-                            NULL,
-                            G_CALLBACK (toggle_decorated_cb),
-                            window, /* user data for callback */
-                            -1);  /* -1 means "append" */
-  
-  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
-                            GTK_STOCK_OPEN,
-                            "This is a demo button that locks the aspect ratio using a hint",
-                            NULL,
-                            G_CALLBACK (toggle_aspect_ratio),
-                            contents, /* user data for callback */
-                            -1);  /* -1 means "append" */
-  
-  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
-                            GTK_STOCK_QUIT,
-                            "This is a demo button with a 'quit' icon",
-                            NULL,
-                            G_CALLBACK (clicked_toolbar_cb),
-                            window, /* user data for callback */
-                            -1);  /* -1 means "append" */
+  insert_stock_button (toolbar, GTK_STOCK_OPEN,
+                       "This is a demo button that locks up the demo",
+                       G_CALLBACK (sleep_cb), window);
+
+  insert_stock_button (toolbar, GTK_STOCK_OPEN,
+                       "This is a demo button that toggles window decorations",
+                       G_CALLBACK (toggle_decorated_cb), window);
+
+  insert_stock_button (toolbar, GTK_STOCK_OPEN,
+                       "This is a demo button that locks the aspect ratio using a hint",
+                       G_CALLBACK (toggle_aspect_ratio), contents);
+
+  insert_stock_button (toolbar, GTK_STOCK_QUIT,
+                       "This is a demo button with a 'quit' icon",
+                       G_CALLBACK (clicked_toolbar_cb), window);
 
   handlebox = gtk_handle_box_new ();
 
